#!/usr/bin/env python3
from collections import namedtuple

import numpy as np
from PIL import Image
from colorspacious import cspace_convert


def is_outline(r: int, g: int, b: int, a: int) -> bool:
    # returns true if a pixel is transparent or pure black
    return a == 0 or (r, g, b) == (0, 0, 0)


def x_metric(pixels: np.array) -> float:
    # X metric - the mean squared Euclidean norm
    # computed as the sum of the squares of the components of the pixels,
    # normalized by the number of pixels
    return sum(sum(pixels ** 2)) / len(pixels)


def y_metric(pixels: np.array) -> np.array:
    # Y metric - the mean pixel of the image
    return sum(pixels) / len(pixels)


ImageInfo = namedtuple(
    "ImageInfo", ["name", "xrgb", "xcam", "yrgb", "ycam"]
)


def ingest_png(file_name: str) -> ImageInfo:
    print(f"Ingesting {file_name}")

    # image name - strip leading path and trailing extension
    name = file_name.rsplit("/", maxsplit=1)[1].split(".", maxsplit=1)[0]

    # read non-outline pixels of image
    rgb_pixels = np.array([
        (r, g, b)
        for r, g, b, a in Image.open(file_name).convert("RGBA").getdata()
        if not is_outline(r, g, b, a)
    ])

    # convert RGB pixels to CAM02 values
    cam_pixels = cspace_convert(rgb_pixels, "sRGB255", "CAM02-UCS")

    # compute and return metrics
    return ImageInfo(
        name=name,
        xrgb=x_metric(rgb_pixels),
        xcam=x_metric(cam_pixels),
        yrgb=y_metric(rgb_pixels),
        ycam=y_metric(cam_pixels),
    )


if __name__ == "__main__":
    import csv
    import os
    import sys

    dir = "pngs" if len(sys.argv) < 2 else sys.argv[1]

    data = [
        ingest_png(dir + "/" + fn)
        for f in os.listdir(dir)
        if (fn := os.fsdecode(f)).endswith(".png")
    ]

    with open("database.csv", "w") as outfile:
        writer = csv.writer(outfile, delimiter=",", quotechar="'")
        writer.writerows([d.name, d.xrgb, *d.yrgb] for d in data)

    with open("database-cam02.csv", "w") as outfile:
        writer = csv.writer(outfile, delimiter=",", quotechar="'")
        writer.writerows([d.name, d.xcam, *d.ycam] for d in data)

    with open("database.js", "w") as outfile:
        outfile.write("const database = [\n")
        for info in data:
            fields = ", ".join(
                (
                    f'name: "{info.name}"',
                    f"xRGB: {info.xrgb}",
                    f"xCAM02: {info.xcam}",
                    f"yRGB: [ {', '.join(str(c) for c in info.yrgb)} ]",
                    f"yCAM02: [ {', '.join(str(c) for c in info.ycam)} ]",
                )
            )
            outfile.write(f"  {{ {fields} }},\n")
        outfile.write("];\n")
